from pecan.rest import RestController
from pecan import request, response
from oslo_log import log as logging
from wsme.rest.json import fromjson as to_model, tojson as from_model
from wsmeext.pecan import wsexpose as expose

from collections import defaultdict
from magellan.db import api as db_api
from magellan.common.consts import ExecutionStatus

from magellan.api.controllers.v1.resources import Device, Execution
from magellan.api.controllers.v1.usertypes import GUID

LOG = logging.getLogger(__name__)


class DevicesController(RestController):

    def __init__(self, rpc_client):
        super().__init__()
        self.rpc_client = rpc_client

    @expose([Device], GUID())
    def get_all(self, execution_id):
        """Return a list of auto discovery devices."""

        LOG.info("Fetch all devices for execution %s." % execution_id)
        devices = self.rpc_client.device_get_all(execution_id)
        connections = self.rpc_client.connection_get_all(execution_id)

        id_address_map = {}
        for device in devices:
            id_address_map[device.id] = device.address

        address_to_neighbour_map = defaultdict(set)
        for connection in connections:
            d1 = connection.device1_id
            d2 = connection.device2_id
            address_to_neighbour_map[d1].add(d2)
            address_to_neighbour_map[d2].add(d1)

        for device in devices:
            device.neighbours = list(map(id_address_map.get,
                                         address_to_neighbour_map[device.id]))

        return [to_model(Device, {
            "address": device.address,
            "neighbours": device.neighbours,
            "details": device.details
        }) for device in devices]


class ExecutionsController(RestController):

    def __init__(self, rpc_client):
        super().__init__()
        self.rpc_client = rpc_client
        self.devices = DevicesController(rpc_client)

    @expose(Execution, GUID())
    def get(self, execution_id):
        """Return the execution.
        :param execution_id: Id of execution to retrieve
        """
        LOG.info("Fetch execution [id=%s]", execution_id)
        db_model = self.rpc_client.execution_get(execution_id)
        return to_model(Execution, db_model)

    @expose(Execution, body=Execution, status_code=201)
    def post(self, new_resource):
        """Create a new execution."""

        LOG.info("Start new execution [%s]", str(new_resource))
        raw_execution = from_model(Execution, new_resource)

        raw_execution['status'] = ExecutionStatus.running
        execution_db = self.rpc_client.execution_create(raw_execution)
        response.headers["Location"] = "%s/%s" % (
            request.url,
            execution_db["id"]
        )

        return to_model(Execution, execution_db)

    @expose(None, GUID(), status_code=204)
    def delete(self, execution_id):
        """Delete the execution.
        :param execution_id: Name of workbook to delete
        """
        LOG.info("Delete execution [id=%s]", execution_id)
        self.rpc_client.execution_delete(execution_id)

    @expose([Execution])
    def get_all(self):
        """Return a list of auto discovery executions."""
        LOG.info("Fetch all executions.")
        return [to_model(Execution, db_model) for db_model in
                db_api.execution_get_all()]
