from wsme import types, wsattr, exc as wsexc
from wsme.rest.json import fromjson

from magellan.api.controllers.v1 import usertypes
from magellan.common import consts


class Job(types.Base):
    """Job resource."""

    id = usertypes.GUID()

    name = wsattr(types.text, mandatory=True)
    description = wsattr(types.text, mandatory=True)
    method = wsattr(usertypes.Enum(consts.ScanMethod), mandatory=True)
    options = wsattr('SeedRouterOptions', mandatory=True)
    timeout = types.IntegerType()
    schedule = types.text
    latest_execution = wsattr('Execution', default=None)

    created_at = usertypes.DateTime()
    updated_at = usertypes.DateTime()

    @classmethod
    def sample(cls):
        return cls(
            id=usertypes.GUID().frombasetype(
                '123e4567-e89b-12d3-a456-426655440000'
            ),
            name='job',
            description='This is a long and useless'
            'description of an auto discovery job',
            method=consts.ScanMethod.seed_router,
            options=SeedRouterOptions.sample(),
            timeout=3600,
            created_at=usertypes.DateTime().frombasetype(
                '2017-01-01 00:00:00'
            ),
            updated_at=usertypes.DateTime().frombasetype(
                '2017-01-01 00:00:00'
            )
        )


class SNMPv1(types.Base):
    version = usertypes.Constant(
        usertypes.Enum(consts.SnmpVersion), consts.SnmpVersion.v1)
    community = wsattr(types.text, default="public")


class SNMPv2(types.Base):
    version = usertypes.Constant(
        usertypes.Enum(consts.SnmpVersion), consts.SnmpVersion.v2c)
    community = wsattr(types.text, default="public")


class SNMPv3AuthPriv(types.Base):
    version = usertypes.Constant(
        usertypes.Enum(consts.SnmpVersion), consts.SnmpVersion.v3)
    secname = wsattr(types.text, mandatory=True)
    authpass = wsattr(types.text, mandatory=True)
    privpass = wsattr(types.text, mandatory=True)


class SNMPv3AuthNoPriv(types.Base):
    version = usertypes.Constant(
        usertypes.Enum(consts.SnmpVersion), consts.SnmpVersion.v3)
    secname = wsattr(types.text, mandatory=True)
    authpass = wsattr(types.text, mandatory=True)


class SNMPv3NoAuthNoPriv(types.Base):
    version = usertypes.Constant(
        usertypes.Enum(consts.SnmpVersion), consts.SnmpVersion.v3)
    secname = wsattr(types.text, mandatory=True)


class SeedRouterOptions(types.Base):
    """Valid seed router options."""

    seed_router = wsattr(usertypes.IPAddress(), mandatory=True)
    snmp = usertypes.Union(
        "SNMP Options", SNMPv3NoAuthNoPriv,
        SNMPv3AuthNoPriv, SNMPv3AuthPriv, SNMPv2, SNMPv1)

    @classmethod
    def sample(cls):
        return cls(seed_router="127.0.0.1",
                   snmp_version=consts.SnmpVersion.v2c,
                   snmp_community="public")


class Execution(types.Base):
    """Execution resource."""

    id = usertypes.GUID()
    job_id = wsattr(usertypes.GUID(), mandatory=True)

    name = wsattr(types.text, mandatory=True)
    description = wsattr(types.text, mandatory=True)
    status = usertypes.Enum(consts.ExecutionStatus)

    complete_time = usertypes.DateTime()
    created_at = usertypes.DateTime()
    updated_at = usertypes.DateTime()

    @classmethod
    def sample(cls):
        return cls(
            id=usertypes.GUID().frombasetype(
                '123e4567-e89b-12d3-a456-426655440000'
            ),
            job_id=usertypes.GUID().frombasetype(
                '123e4567-e89b-12d3-a456-426655440001'
            ),
            name='execution',
            description='This is a long and useless'
            'description of an auto discovery execution',
            status=consts.ExecutionStatus.canceled,
            complete_time=usertypes.DateTime().frombasetype(
                '2017-01-01 00:00:00'
            ),
            created_at=usertypes.DateTime().frombasetype(
                '2017-01-01 00:00:00'
            ),
            updated_at=usertypes.DateTime().frombasetype(
                '2017-01-01 00:00:00'
            )
        )


class Device(types.Base):
    """Network device."""

    address = types.text
    neighbours = [types.text]
    details = {
        str: str
    }

    @classmethod
    def sample(cls):
        return cls(
            address="127.0.0.1",
            neighbours=["192.168.0.1", "192.168.0.2"],
            details={
                "sysName": "MyRouter"
            }
        )


@fromjson.when_object(Job)
@fromjson.when_object(SeedRouterOptions)
@fromjson.when_object(Execution)
def try_fromjson(datatype, value):
    """
    This function adds a check when we convert input data to our defined WSME
    types. The generic fromjson implementation assumes that value is a
    dict-like object and does not handle the exception if it isn't.

    This means we get a server error if the user supplies a valid JSON
    datatype other than a JSON object for these types. We want that case to
    raise a InvalidInput exception instead.

    We can't directly check if value is a dict because it can also be a
    SqlAlchemy object (which behaves like a dict).

    It looks like this is a bug in WSME, but it could also be that we simply
    don't define our types correctly.
    """
    if value is None:
        return value
    if hasattr(value, "keys"):
        return fromjson.default(datatype, value)
    raise wsexc.InvalidInput(datatype.__name__.lower(), value)
