import json
import uuid
import enum
import datetime

from wsme import exc as wsme_exc, types as wtypes
from wsme.rest import json as wjson
from magellan.common import exceptions as exc
from ipaddress import AddressValueError, IPv4Address


class JsonType(wtypes.UserType):
    """A simple JSON type."""

    basetype = wtypes.text
    name = 'json'

    def validate(self, value):
        if not value:
            return {}
        try:
            return json.loads(value)
        except ValueError as e:
            raise ValueError(e)

    def frombasetype(self, value):
        if isinstance(value, dict):
            return value
        try:
            return json.loads(value) if value is not None else None
        except TypeError as e:
            raise ValueError(e)

    def tobasetype(self, value):
        return value


class IPAddress(wtypes.UserType):
    """A simple IPv4 Address type."""

    basetype = wtypes.text
    name = 'IPAddress'

    def frombasetype(self, value):
        try:
            return IPv4Address(value)
        except (AddressValueError, ValueError) as e:
            raise ValueError('Invalid IPv4 address {}'.format(value))

    def tobasetype(self, value):
        return str(value)


class GUID(wtypes.UserType):
    """A simple GUID type."""

    basetype = wtypes.text
    name = 'guid'

    def validate(self, value):
        if not value:
            return uuid.uuid4()

        if not isinstance(value, uuid.UUID):
            raise exc.InputException(
                'GUID field value must be a valid UUIDv4 [actual=%s]' % value
            )

        return value

    def frombasetype(self, value):
        if isinstance(value, uuid.UUID):
            return value
        try:
            return uuid.UUID(value)
        except TypeError as e:
            raise ValueError(e)

    def tobasetype(self, value):
        return str(value)


class Enum(wtypes.UserType):
    """A simple PythonEnum type."""

    def __init__(self, enumeration):
        self.enumeration = enumeration
        self.basetype = enum.Enum

    def validate(self, value):
        if isinstance(value, str) and value in self.enumeration.__members__:
            return self.enumeration[value]
        elif isinstance(value, self.enumeration):
            return value
        else:
            raise ValueError("Invalid in Enum")

    def frombasetype(self, value):
        return self.validate(value)

    def tobasetype(self, value):
        instance = self.validate(value)
        return instance.name


class Union(wtypes.UserType):
    """A simple Disjoint Union/Set Type/Variant Record of WSME Models."""

    basetype = wtypes.Base

    def __init__(self, name, *types):
        self.name = name
        self.types = types

    def frombasetype(self, value):
        for T in self.types:
            try:
                return wjson.fromjson(T, value)
            except (exc.InputException,
                    wsme_exc.UnknownAttribute,
                    wsme_exc.InvalidInput,
                    AttributeError,
                    ValueError,
                    TypeError) as e:
                pass
        raise ValueError(
            '{} does not match any variant of {}'.format(value, self.name))

    def tobasetype(self, value):
        return wjson.tojson(type(value), value)


class Constant(wtypes.UserType):

    basetype = wtypes.text

    def __init__(self, T, value):
        self.T = T
        self.value = T.frombasetype(value)

    def frombasetype(self, value):
        try:
            validated = self.T.validate(value)
            if self.value == validated:
                return self.value
            else:
                raise ValueError('Invalid value {} for expected {}'.format(
                    value, self.value))
        except ValueError as e:
            raise ValueError(
                '{} did not match constant value {} of type {}'.format(
                    value, self.value, type(self.T)))

    def tobasetype(self, value):
        return self.T.tobasetype(value)


class DateTime(wtypes.UserType):
    """A simple Datetime type."""

    basetype = wtypes.datetime
    name = 'datetime'

    def validate(self, value):
        if not isinstance(value, datetime.datetime):
            raise exc.InputException(
                'Datetime field value must be a valid datetime '
                '[actual=%s]' % value
            )
        return value

    def frombasetype(self, value):
        if isinstance(value, datetime.datetime):
            return value
        try:
            return datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
        except TypeError as e:
            raise ValueError(e)

    def tobasetype(self, value):
        return str(value)
