"""Routines for configuring Magellan."""
from oslo_config import cfg
from oslo_log import log
from oslo_middleware import cors

from magellan import version

# Options under default group.
launch_opt = cfg.ListOpt(
    'server',
    default=['all'],
    help=('Specifies which magellan server to start by the launch script. '
          'Valid options are all or any combination of '
          'api and engine.')
)

api_opts = [
    cfg.HostAddressOpt(
        'host',
        default='0.0.0.0',
        help='Magellan API server host'
    ),
    cfg.PortOpt('port', default=8008, help='Magellan API server port'),
    cfg.BoolOpt(
        'enable_ssl_api',
        default=False,
        help=('Enable the integrated stand-alone API to service requests'
              'via HTTPS instead of HTTP.')
    ),
    cfg.IntOpt(
        'api_workers',
        help=('Number of workers for Magellan API service '
              'default is equal to the number of CPUs available if that can '
              'be determined, else a default worker count of 1 is returned.')
    )
]

pecan_opts = [
    cfg.StrOpt(
        'root',
        default='magellan.api.controllers.root.RootController',
        help='Pecan root controller'
    ),
    cfg.ListOpt(
        'modules',
        default=["magellan.api"],
        help='A list of modules where pecan will search for applications.'
    ),
    cfg.BoolOpt(
        'debug',
        default=False,
        help=('Enables the ability to display tracebacks in the browser and'
              ' interactively debug during development.')
    ),
    cfg.BoolOpt(
        'auth_enable',
        default=False,
        help='Enables user authentication in pecan.'
    )
]

engine_opts = [
    cfg.HostAddressOpt(
        'host',
        default='0.0.0.0',
        help=('Name of the engine node. This can be an opaque '
              'identifier. It is not necessarily a hostname, '
              'FQDN, or IP address.')
    ),
    cfg.StrOpt('version', default='1.0', help='The version of the engine.'),
    cfg.IntOpt(
        'engine_workers',
        help=('Number of workers for Magellan RPC server '
              'default is equal to the number of CPUs available if that can '
              'be determined, else a default worker count of 1 is returned.')
    )
]

messaging_opts = [
    cfg.StrOpt(
        'transport-url',
        default=None,
        help='The URL to the RabbitMQ server.'
    ),
    cfg.StrOpt(
        'exchange',
        default='magellan',
        help='The control exchange name used for the rpc transport layer.'
    )
]

CONF = cfg.CONF

API_GROUP = 'api'
PECAN_GROUP = 'pecan'
ENGINE_GROUP = 'engine'
MESSAGING_GROUP = 'messaging'

CONF.register_opts(api_opts, group=API_GROUP)
CONF.register_opts(pecan_opts, group=PECAN_GROUP)
CONF.register_opts(engine_opts, group=ENGINE_GROUP)
CONF.register_opts(messaging_opts, group=MESSAGING_GROUP)

CONF.register_cli_opts([
    launch_opt
])

_DEFAULT_LOG_LEVELS = [
    'eventlet.wsgi.server=WARN',
    'oslo_service.periodic_task=INFO',
    'oslo_service.loopingcall=INFO',
    'oslo_db=WARN'
]


def list_opts():
    yield API_GROUP, api_opts
    yield PECAN_GROUP, pecan_opts
    yield ENGINE_GROUP, engine_opts
    yield MESSAGING_GROUP, messaging_opts
    yield None, messaging_opts


def parse_args(args=None, usage=None, default_config_files=None):
    default_log_levels = log.get_default_log_levels()
    default_log_levels.extend(_DEFAULT_LOG_LEVELS)
    log.set_defaults(default_log_levels=default_log_levels)

    log.register_options(CONF)

    CONF(
        args=args,
        project='magellan',
        version=version,
        usage=usage,
        default_config_files=default_config_files
    )


def set_config_defaults():
    """This method updates all configuration default values."""
    set_cors_middleware_defaults()


def set_cors_middleware_defaults():
    """Update default configuration options for oslo.middleware."""
    cors.set_defaults(
        allow_methods=['GET',
                       'PUT',
                       'POST',
                       'DELETE',
                       'PATCH']
    )
