class MagellanError(Exception):
    """Magellan specific error.
    Reserved for situations that can't be automatically handled. When it occurs
    it signals that there is a major environmental problem like invalid startup
    configuration or implementation problem (e.g. some code doesn't take care
    of certain corner cases). From architectural perspective it's pointless to
    try to handle this type of problems except doing some finalization work
    like transaction rollback, deleting temporary files etc.
    """

    message = "An unknown error occurred"
    http_code = 500

    def __init__(self, message=None):
        if message is not None:
            self.message = message

        super(MagellanError, self).__init__(
            '%d: %s' % (self.http_code, self.message))

    @property
    def code(self):
        return self.http_code

    def __str__(self):
        return self.message


class MagellanException(Exception):
    """Magellan specific exception.
    Reserved for situations that are not critical for program continuation.
    It is possible to recover from this type of problems automatically and
    continue program execution. Such problems may be related with invalid user
    input (such as invalid syntax) or temporary environmental problems.
    In case if an instance of a certain exception type bubbles up to API layer
    then this type of exception it must be associated with an http code so it's
    clear how to represent it for a client.
    To correctly use this class, inherit from it and define a 'message' and
    'http_code' properties.
    """
    message = "An unknown exception occurred"
    http_code = 500

    def __init__(self, message=None):
        if message is not None:
            self.message = message

        super(MagellanException, self).__init__(
            '%d: %s' % (self.http_code, self.message))

    @property
    def code(self):
        return self.http_code

    def __str__(self):
        return self.message


# Database errors.

class DBError(MagellanError):
    http_code = 400


class DBDuplicateEntryError(DBError):
    http_code = 409
    message = "Database object already exists"


class DBQueryEntryError(DBError):
    http_code = 400


class DBEntityNotFoundError(DBError):
    http_code = 404
    message = "Object not found"


class JobAlreadyRunningError(MagellanException):
    http_code = 503
    message = "There is already an Execution running for the given Job"


class InputException(MagellanException):
    http_code = 400
