'''
Interface for database access.

SQLAlchemy is currently the only supported backend.
'''

from oslo_config import cfg
from oslo_db import api

from magellan.common.consts import ExecutionStatus
from magellan.common.exceptions import JobAlreadyRunningError
from magellan.engine.discover import execution_start

CONF = cfg.CONF

_BACKEND_MAPPING = {'sqlalchemy': 'magellan.db.sqlalchemy.api'}

IMPL = api.DBAPI.from_config(CONF, backend_mapping=_BACKEND_MAPPING)


def get_engine():
    return IMPL.get_engine()


def get_session():
    return IMPL.get_session()


def db_sync(engine, version=None):
    """Migrate the database to `version` or the most recent version."""
    return IMPL.db_sync(engine, version=version)


def db_version(engine):
    """Display the current database version."""
    return IMPL.db_version(engine)


def job_create(values):
    """Create an auto discovery job."""
    return IMPL.job_create(values)


def job_get(job_id):
    """Retrieve an auto discovery job or raise if it does not exist."""
    return IMPL.job_get(job_id)


def job_get_all():
    """Retrieve all registered jobs."""
    return IMPL.job_get_all()


def job_update(job_id, values):
    """Update a job or raise if it does not exist."""
    return IMPL.job_update(job_id, values)


def job_delete(job_id):
    """Delete a job or raise if it does not exist."""
    return IMPL.job_delete(job_id)


def execution_create(values):
    """Create an auto discovery job."""
    # Make sure that an execution is not started while an execution is already
    # running for the same job
    job_executions = execution_get_all_by_job(values['job_id'])
    if len(job_executions) > 0:
        if job_executions[0].status == ExecutionStatus.running:
            raise JobAlreadyRunningError()

    execution = IMPL.execution_create(values)
    execution_start(execution)
    return execution


def execution_get(execution_id):
    """Retrieve an auto discovery execution or raise if it does not exist."""
    return IMPL.execution_get(execution_id)


def execution_get_all():
    """Retrieve all registered executions."""
    return IMPL.execution_get_all()


def execution_get_all_by_job(job_id):
    """Retrieve all registered executions for job."""
    return IMPL.execution_get_all_by_job(job_id)


def execution_cancel(execution_id):
    """Cancel an execution or raise if it does not exist."""
    return IMPL.execution_cancel(execution_id)


def execution_update(execution_id, values):
    """Update an execution or raise if it does not exist."""
    return IMPL.execution_update(execution_id, values)


def execution_delete(execution_id):
    """Delete an execution or raise if it does not exist."""
    return IMPL.execution_delete(execution_id)


def device_get_all(execution_id):
    """Get all devices for an execution_id"""
    return IMPL.device_get_all(execution_id)


def connection_get_all(execution_id):
    """Get all connections for an execution_id"""
    return IMPL.connection_get_all(execution_id)


def device_create(values):
    """Create an auto discovery device."""
    return IMPL.device_create(values)


def connection_create(values):
    """Create an auto discovery connection."""
    return IMPL.connection_create(values)
