"""
Implementation of SQLAlchemy backend.
"""

import sys
import threading
import functools

from oslo_db.sqlalchemy import enginefacade
from oslo_db.sqlalchemy.exc_filters import exception as sqlalchemy_exceptions
from oslo_log import log as logging

from magellan.common import exceptions as exception
from magellan.db.sqlalchemy import migration
from magellan.db.sqlalchemy import models

LOG = logging.getLogger(__name__)

_main_context_manager = None
_CONTEXT = threading.local()


def _get_main_context_manager():
    global _main_context_manager
    if not _main_context_manager:
        _main_context_manager = enginefacade.transaction_context()
    return _main_context_manager


def get_engine():
    return _get_main_context_manager().get_legacy_facade().get_engine()


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def read_session():
    return _get_main_context_manager().reader.using(_CONTEXT)


def write_session():
    return _get_main_context_manager().writer.using(_CONTEXT)


def db_sync(engine, version=None):
    """Migrate the database to `version` or the most recent version."""
    return migration.db_sync(engine, version=version)


def db_version(engine):
    """Display the current database version."""
    return migration.db_version(engine)


def translate_exceptions(func):
    """
    Decorator for translating SQL Alchemy exceptions into Magellan exceptions.
    """
    @functools.wraps(func)
    def wrapped(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except sqlalchemy_exceptions.DBDuplicateEntry:
            raise exception.DBDuplicateEntryError()

    return wrapped


@translate_exceptions
def job_get(job_id):
    """Get an auto discovery job by id."""
    with read_session() as session:
        job = session.query(models.AutoDiscoveryJob).filter(
            models.AutoDiscoveryJob.id == job_id).one_or_none()
        if not job:
            raise exception.DBEntityNotFoundError()
        return job


@translate_exceptions
def job_get_all():
    """Get all available auto discovery jobs."""
    with read_session() as session:
        jobs = session.query(models.AutoDiscoveryJob).all()
        return jobs


@translate_exceptions
def job_create(values):
    """
    Create a new auto discovery job.
    :param values: a job structure.
    :return: a created job object.
    """
    with write_session() as session:
        job = models.AutoDiscoveryJob()
        job.update(values.copy())
        session.add(job)
        return job


@translate_exceptions
def job_update(job_id, values):
    """
    Update an existing auto discovery job.
    :param job_id: id of a job to update.
    :param values: new job parameters.
    :return: an updated job object
    """
    with write_session() as session:
        job = job_get(job_id)
        job.update(values.copy())
        job.save(session)
        return job


@translate_exceptions
def job_delete(job_id):
    """
    Delete a job.
    :param job_id: the id of a job to delete.
    """
    with write_session() as session:
        job = job_get(job_id)
        session.delete(job)


@translate_exceptions
def execution_get(execution_id):
    """
    Get an auto discovery execution by id.
    :param execution_id: The id of the execution to get
    :return: Instance of the execution
    """
    with read_session() as session:
        execution = session.query(models.AutoDiscoveryExecution).filter(
            models.AutoDiscoveryExecution.id == execution_id
        ).one_or_none()
        if not execution:
            raise exception.DBEntityNotFoundError()
        return execution


@translate_exceptions
def execution_get_all():
    """
    Get all available auto discovery executions.
    :return: List of all execution instances
    """
    with read_session() as session:
        executions = session.query(models.AutoDiscoveryExecution).all()
        return executions


@translate_exceptions
def execution_get_all_by_job(job_id):
    """
    Get all available auto discovery executions.
    :param job_id: The id of the job to retrieve executions for
    :return: List of all execution instances for a job
    """
    job_get(job_id)
    with read_session() as session:
        executions = session.query(models.AutoDiscoveryExecution).filter(
            models.AutoDiscoveryExecution.job_id == job_id
        ).order_by(models.AutoDiscoveryExecution.created_at.desc()).all()
        return executions


@translate_exceptions
def execution_create(values):
    """
    Create a new auto discovery execution.
    :param values: a execution structure.
    :return: a created execution object.
    """
    try:
        with write_session() as session:
            execution = models.AutoDiscoveryExecution()
            execution.update(values.copy())
            session.add(execution)
            return execution
    except sqlalchemy_exceptions.DBReferenceError:
        raise exception.DBEntityNotFoundError()


@translate_exceptions
def execution_update(execution_id, values):
    """
    Update an existing auto discovery execution.
    :param execution_id: id of a execution to update.
    :return: a canceled execution object
    """
    with write_session() as session:
        execution = session.query(models.AutoDiscoveryExecution).filter(
            models.AutoDiscoveryExecution.id == execution_id
        ).one_or_none()
        if not execution:
            raise exception.DBEntityNotFoundError()

        execution.update(values)
        execution.save(session)

        return execution


@translate_exceptions
def execution_cancel(execution_id):
    """
    Update an existing auto discovery execution.
    :param execution_id: id of a execution to update.
    :return: a canceled execution object
    """
    with write_session() as session:
        execution = execution_get(execution_id)
        execution.update({"status": "canceled"})
        execution.save(session)
        return execution


@translate_exceptions
def execution_delete(execution_id):
    """
    Delete an execution.
    :param execution_id: the id of a execution to delete.
    """
    with write_session() as session:
        execution = execution_get(execution_id)
        session.delete(execution)


@translate_exceptions
def device_get_all(execution_id):
    """
    Get all available auto discovery devices for an execution.
    :return: List of all devices for an execution
    """
    with read_session() as session:

        execution = session.query(models.AutoDiscoveryExecution).filter(
            models.AutoDiscoveryExecution.id == execution_id
        ).one_or_none()

        if execution is None:
            raise exception.DBEntityNotFoundError()

        return execution.devices


@translate_exceptions
def connection_get_all(execution_id):
    """
    Get all available auto discovery connections for an execution.
    :return: List of all connections for an execution
    """
    with read_session() as session:

        execution = session.query(models.AutoDiscoveryExecution).filter(
            models.AutoDiscoveryExecution.id == execution_id
        ).one_or_none()

        if execution is None:
            raise exception.DBEntityNotFoundError()

        return execution.connections


@translate_exceptions
def device_create(values):
    """
    Create a new auto discovery device.
    :param values: a device structure.
    :return: a created device object.
    """
    with write_session() as session:
        device = models.AutoDiscoveryDevice()
        device.update(values.copy())
        session.add(device)
        return device


@translate_exceptions
def connection_create(values):
    """
    Create a new auto discovery connection.
    :param values: a connection structure.
    :return: a created connection object.
    """
    with write_session() as session:
        connection = models.AutoDiscoveryConnection()
        connection.update(values.copy())
        session.add(connection)
        return connection
