import sqlalchemy as sa

from magellan.db.sqlalchemy.types import GUID, JSONEncodedDict
from magellan.common.consts import ScanMethod, ExecutionStatus


def upgrade(migrate_engine):
    meta = sa.MetaData()
    meta.bind = migrate_engine

    discovery_jobs = sa.Table(
        'jobs', meta,
        sa.Column('id', GUID,
                  nullable=False, primary_key=True),
        sa.Column('name', sa.String(255),
                  unique=True, nullable=False),
        sa.Column('description', sa.String(1023)),
        sa.Column('method', sa.Enum(ScanMethod),
                  nullable=False),
        sa.Column('options', JSONEncodedDict(1023),
                  nullable=False),
        sa.Column('timeout', sa.Integer,
                  nullable=False, default=3600),
        sa.Column('schedule', sa.String(255)),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    discovery_executions = sa.Table(
        'executions', meta,
        sa.Column('id', GUID,
                  primary_key=True, nullable=False),
        sa.Column('job_id', GUID,
                  sa.ForeignKey('jobs.id', ondelete='SET NULL')),
        sa.Column('name', sa.String(255),
                  nullable=False),
        sa.Column('description', sa.String(1023),
                  nullable=True),
        sa.Column('status', sa.Enum(ExecutionStatus),
                  nullable=False),
        sa.Column('complete_time', sa.DateTime),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    discovery_devices = sa.Table(
        'devices', meta,
        sa.Column('id', GUID,
                  nullable=False, primary_key=True),
        sa.Column('execution_id', GUID,
                  sa.ForeignKey('executions.id', ondelete='CASCADE'),
                  nullable=False),
        sa.Column('address', sa.String(14), nullable=False),
        sa.Column('details', JSONEncodedDict(1023), nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    discovery_connections = sa.Table(
        'connections', meta,
        sa.Column('id', GUID,
                  nullable=False, primary_key=True),
        sa.Column('execution_id', GUID,
                  sa.ForeignKey('executions.id'),
                  nullable=False),
        sa.Column('device1_id', GUID,
                  sa.ForeignKey('devices.id', ondelete='CASCADE'),
                  nullable=False),
        sa.Column('device2_id', GUID,
                  sa.ForeignKey('devices.id', ondelete='CASCADE'),
                  nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('deleted_at', sa.DateTime),
        mysql_engine='InnoDB',
        mysql_charset='utf8'
    )

    tables = (
        discovery_jobs,
        discovery_executions,
        discovery_devices,
        discovery_connections,
    )

    for index, table in enumerate(tables):
        try:
            table.create()
        except Exception:
            # If an error occurs, drop all tables created so far to return
            # to the previously existing state.
            meta.drop_all(tables=tables[:index])
            raise


def downgrade(migrate_engine):
    raise NotImplementedError('Database downgrade not supported - '
                              'would drop all tables')
