"""
SQLAlchemy models for discovery data.
"""

from oslo_db.sqlalchemy import models
import uuid

from sqlalchemy.orm import session as orm_session, relationship
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, Enum
from sqlalchemy.ext.declarative import declarative_base

from magellan.common.consts import ScanMethod, ExecutionStatus
from magellan.db.sqlalchemy.types import GUID, JSONEncodedDict

BASE = declarative_base()


def get_session():
    from magellan.db.sqlalchemy import api as db_api

    return db_api.get_session()


class DiscoveryBase(models.ModelBase,
                    models.TimestampMixin):
    """Base class for Discovery Models."""

    __table_args__ = {'mysql_engine': 'InnoDB'}

    def expire(self, session=None, attrs=None):
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.expire(self, attrs)

    def refresh(self, session=None, attrs=None):
        """Refresh this object."""
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.refresh(self, attrs)

    def delete(self, session=None):
        """Delete this object."""
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.begin()
        session.delete(self)
        session.commit()


class AutoDiscoveryJob(BASE, DiscoveryBase):
    __tablename__ = "jobs"

    id = Column(GUID, primary_key=True, default=uuid.uuid4, nullable=False)
    name = Column(String(255), unique=True, nullable=False)
    description = Column(String(1023))
    method = Column(Enum(ScanMethod), nullable=False)
    options = Column(JSONEncodedDict(1023), nullable=False)
    timeout = Column(Integer, default=3600, nullable=False)
    schedule = Column(String(255))
    executions = relationship('AutoDiscoveryExecution', backref="job")


class AutoDiscoveryExecution(BASE, DiscoveryBase):
    __tablename__ = "executions"

    id = Column(GUID, primary_key=True, default=uuid.uuid4, nullable=False)
    job_id = Column(GUID, ForeignKey('jobs.id'))
    devices = relationship(
        'AutoDiscoveryDevice',
        backref="execution",
        cascade="all, delete-orphan",
        passive_deletes=True
    )
    connections = relationship(
        'AutoDiscoveryConnection',
        backref="execution",
        cascade="all, delete-orphan",
        passive_deletes=True
    )
    name = Column(String(255), nullable=False)
    description = Column(String(1023))
    status = Column(Enum(ExecutionStatus), nullable=False)
    complete_time = Column(DateTime)


class AutoDiscoveryDevice(BASE, DiscoveryBase):
    __tablename__ = "devices"

    id = Column(GUID, primary_key=True, default=uuid.uuid4, nullable=False)
    execution_id = Column(GUID, ForeignKey('executions.id'), nullable=False)
    address = Column(String(14), nullable=False)
    details = Column(JSONEncodedDict(1023), nullable=False)


class AutoDiscoveryConnection(BASE, DiscoveryBase):
    __tablename__ = "connections"

    id = Column(GUID, primary_key=True, default=uuid.uuid4, nullable=False)
    execution_id = Column(GUID, ForeignKey('executions.id'), nullable=False)
    device1_id = Column(GUID, ForeignKey('devices.id'), nullable=False)
    device2_id = Column(GUID, ForeignKey('devices.id'), nullable=False)
