import datetime
import ipaddress

from oslo_db.exception import DBReferenceError
from oslo_log import log as logging

from magellan.common.consts import ScanMethod, ExecutionStatus
from magellan.common.exceptions import DBEntityNotFoundError
from magellan.engine.threadmanager import ThreadManager
from magellan import db
from pyautodiscover import discover, info
from pyautodiscover.backends.snmp import (
    SNMPBackend, PrivInfo, PrivProto, AuthInfo, AuthProto, SNMPInfo
)

LOG = logging.getLogger(__name__)

threadmanager = ThreadManager()


def execution_start(execution):
    threadmanager.start(_execution_start, execution)


def _execution_start(execution):
    """ Starts an autodiscovery job based on an execution """

    job = db.api.job_get(execution.job_id)

    try:
        if job.method == ScanMethod.seed_router:

            snmpoptions = job.options['snmp']
            if snmpoptions.get('version') == 'v1':
                snmpinfo = SNMPInfo.SnmpV1(snmpoptions.get('community'))
            elif snmpoptions.get('version') == 'v2c':
                snmpinfo = SNMPInfo.SnmpV2c(snmpoptions.get('community'))
            elif snmpoptions.get('version') == 'v3':
                if snmpoptions.get('authpass') is not None:
                    ai = AuthInfo(AuthProto.SHA(), snmpoptions.get('authpass'))
                    if snmpoptions.get('privpass') is not None:
                        pi = PrivInfo(
                            PrivProto.AES(), snmpoptions.get('privpass'))
                        snmpinfo = SNMPInfo.SnmpV3AuthPriv(
                            snmpoptions.get('secname'), ai, pi)
                    else:
                        snmpinfo = SNMPInfo.SnmpV3AuthNoPriv(
                            snmpoptions.get('secname'), ai)
                else:
                    snmpinfo = SNMPInfo.SnmpV3NoAuthNoPriv(
                        snmpoptions.get('secname'))

            backend = SNMPBackend(snmpinfo)
            links = discover(backend, ipaddress.ip_address(
                job.options.get('seed_router')))
            devices = dict()

            for link in links:

                if link.address_a and link.address_a not in devices:
                    devices[link.address_a] = db.api.device_create({
                        'execution_id': execution.id,
                        'address': str(link.address_a),
                        'details': info(backend, link.address_a)
                    })

                if link.address_b and link.address_b not in devices:
                    devices[link.address_b] = db.api.device_create({
                        'execution_id': execution.id,
                        'address': str(link.address_b),
                        'details': info(backend, link.address_b)
                    })

                if link.address_a in devices and link.address_b in devices:
                    db.api.connection_create({
                        'execution_id': execution.id,
                        'device1_id': devices[link.address_a].id,
                        'device2_id': devices[link.address_b].id
                    })

        db.api.execution_update(execution.id, {
            'status': ExecutionStatus.completed,
            'complete_time': datetime.datetime.utcnow()
        })

    except (DBReferenceError, DBEntityNotFoundError):
        LOG.info(
            'The execution with id {} was deleted while running.'.format(
                execution.id
            )
        )
        return
