from oslo_config import cfg
from oslo_context import context
from oslo_log import log as logging

from magellan.common import consts
from magellan.common import messaging


LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class RpcClient:
    """Client side of the Magellan RPC API."""

    def __init__(self):
        self._client = messaging.get_rpc_client(
            topic=("%s_%s" % (CONF.messaging.exchange, "rpc")),
            version=consts.RPC_API_VERSION
        )

    @staticmethod
    def make_msg(method, **kwargs):
        return method, kwargs

    def call(self, msg, version=None):
        method, kwargs = msg
        if version is not None:
            client = self._client.prepare(version=version)
        else:
            client = self._client
        return client.call(context.RequestContext(), method, **kwargs)

    def cast(self, msg, version=None):
        method, kwargs = msg
        if version is not None:
            client = self._client.prepare(version=version)
        else:
            client = self._client
        return client.cast(context.RequestContext(), method, **kwargs)

    # Job methods

    def job_create(self, values):
        return self.call(self.make_msg('job_create', values=values))

    def job_get_all(self):
        return self.call(self.make_msg('job_get_all'))

    def job_get(self, job_id):
        return self.call(self.make_msg('job_get', job_id=job_id))

    def job_update(self, job_id, values):
        return self.call(
            self.make_msg('job_update', job_id=job_id, values=values))

    def job_delete(self, job_id):
        self.call(self.make_msg('job_delete', job_id=job_id))

    # Execution methods

    def execution_create(self, values):
        return self.call(self.make_msg('execution_create', values=values))

    def execution_delete(self, execution_id):
        return self.call(
            self.make_msg('execution_delete', execution_id=execution_id))

    def execution_cancel(self, execution_id):
        return self.call(
            self.make_msg('execution_cancel', execution_id=execution_id))

    def execution_get(self, execution_id):
        return self.call(
            self.make_msg('execution_get', execution_id=execution_id))

    def execution_get_all(self):
        return self.call(self.make_msg('execution_get_all'))

    def execution_get_all_by_job(self, job_id):
        return self.call(self.make_msg('execution_get_all_by_job',
                                       job_id=job_id))

    # Device methods

    def device_get_all(self, execution_id):
        return self.call(self.make_msg('device_get_all',
                                       execution_id=execution_id))

    # Connection methods

    def connection_get_all(self, execution_id):
        return self.call(self.make_msg('connection_get_all',
                                       execution_id=execution_id))
