import oslo_messaging

from oslo_config import cfg
from oslo_log import log as logging
from magellan.common import consts
from magellan.common import messaging as rpc_messaging
from magellan.common import exceptions
from magellan.db import api as db_api


LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class RpcServer(object):
    """ RPC server.
    This class gets registered as an RPC endpoint for engine specific calls.
    """

    def __init__(self, engine):

        self.engine = engine

        self.host = CONF.engine.host
        self.rpc_api_version = consts.RPC_API_VERSION
        self.topic = "%s_%s" % (CONF.messaging.exchange, "rpc")
        # The following are initialized here, but assigned in start() which
        # happens after the fork when spawning multiple worker processes
        self.target = None
        self._rpc_server = None

    def start(self):
        self.target = oslo_messaging.Target(
            version=self.rpc_api_version,
            server=self.host,
            topic=self.topic
        )
        self._rpc_server = rpc_messaging.get_rpc_server(self.target, self)
        self._rpc_server.start()

        LOG.info('Magellan RPC server started')

    def stop(self):
        try:
            self._rpc_server.stop()
            self._rpc_server.wait()
            LOG.info('Magellan RPC server stopped')
        except Exception as ex:
            LOG.error('Failed to stop Magellan RPC server: %s', str(ex))

    # Job methods

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def job_create(self, ctxt, values):
        LOG.debug("Create job [%s]", values)
        return db_api.job_create(values)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def job_get_all(self, ctxt):
        LOG.debug("Fetch all jobs.")
        return db_api.job_get_all()

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def job_get(self, ctxt, job_id):
        LOG.debug("Fetch job [id=%s]", job_id)
        return db_api.job_get(job_id)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def job_update(self, ctxt, job_id, values):
        LOG.debug("Update job [id=%s]", job_id)
        return db_api.job_update(job_id, values)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def job_delete(self, ctxt, job_id):
        LOG.debug("Delete job [id=%s]", job_id)
        db_api.job_delete(job_id)

    # Execution methods

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_create(self, ctxt, values):
        LOG.debug("Create execution [%s]", values)
        return db_api.execution_create(values)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_delete(self, ctxt, execution_id):
        LOG.debug("Delete execution [id=%s]", execution_id)
        return db_api.execution_delete(execution_id)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_cancel(self, ctxt, execution_id):
        LOG.debug("Cancel execution [id=%s]", execution_id)
        return db_api.execution_cancel(execution_id)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_get(self, ctxt, execution_id):
        LOG.debug("Fetch execution [id=%s]", execution_id)
        return db_api.execution_get(execution_id)

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_get_all(self, ctxt):
        LOG.debug("Fetch all executions")
        return db_api.execution_get_all()

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def execution_get_all_by_job(self, ctxt, job_id):
        LOG.debug("Fetch all executions for job")
        return db_api.execution_get_all_by_job(job_id)

    # Device methods

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def device_get_all(self, ctxt, execution_id):
        LOG.debug("Fetch all devices")
        return db_api.device_get_all(execution_id)

    # Connection methods

    @oslo_messaging.expected_exceptions(
        exceptions.MagellanException,
        exceptions.MagellanError
    )
    def connection_get_all(self, ctxt, execution_id):
        LOG.debug("Fetch all connections")
        return db_api.connection_get_all(execution_id)
