
import unittest

from wsme.rest import json as wjson
from magellan.api.controllers.v1.resources import Job, SNMPv1, SNMPv2, \
                                                  SNMPv3AuthPriv, \
                                                  SNMPv3AuthNoPriv, \
                                                  SNMPv3NoAuthNoPriv


class TestJobResource(unittest.TestCase):

    def test_job_invalid_ip_address(self):
        with self.assertRaisesRegex(
                ValueError, "Invalid IPv4 address 127,0.0.1"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127,0.0.1',
                    'snmp': {
                        'version': 'v3',
                        'secname': 'testuser'
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v1(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v1',
                    'community': 'public'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv1)

    def test_job_snmp_v1_has_default_community(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v1'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv1)
        assert job.options.snmp.community == 'public'

    def test_job_snmp_v2(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v2c',
                    'community': 'public'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv2)

    def test_job_snmp_v2_has_default_community(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v2c'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv2)
        assert job.options.snmp.community == 'public'

    def test_job_snmp_v3(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v3',
                    'secname': 'testuser',
                    'authpass': 'testauthpass',
                    'privpass': 'testprivpass'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv3AuthPriv)

    def test_job_snmp_v3AuthNoPriv(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v3',
                    'secname': 'testuser',
                    'authpass': 'testprivpass'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv3AuthNoPriv)

    def test_job_snmp_v3NoAuthNoPriv(self):
        job = wjson.fromjson(Job, {
            'id': '123e4567-e89b-12d3-a456-426655440000',
            'name': 'job',
            'description': 'This is a long and useless',
            'method': 'seed_router',
            'options': {
                'seed_router': '127.0.0.1',
                'snmp': {
                    'version': 'v3',
                    'secname': 'testuser'
                }
            },
            'timeout': 3600
        })
        assert isinstance(job.options.snmp, SNMPv3NoAuthNoPriv)

    def test_job_snmp_v3_missing_data(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 'v3',
                        'seciname': 'testuser'
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v3_invalid_type_for_secname(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 'v3',
                        'secname': 123
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v3_invalid_value_for_version(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 'v4',
                        'secname': 'testuser'
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v3_invalid_type_for_version(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 1,
                        'secname': 'testuser'
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v3_invalid_type_for_authpass(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 'v3',
                        'secname': 'testuser',
                        'authpass': 123
                    }
                },
                'timeout': 3600
            })

    def test_job_snmp_v3_invalid_type_for_privpass(self):
        with self.assertRaisesRegex(
                ValueError, "does not match any variant of SNMP Options"):
            wjson.fromjson(Job, {
                'id': '123e4567-e89b-12d3-a456-426655440000',
                'name': 'job',
                'description': 'This is a long and useless',
                'method': 'seed_router',
                'options': {
                    'seed_router': '127.0.0.1',
                    'snmp': {
                        'version': 'v3',
                        'secname': 'testuser',
                        'authpass': '123',
                        'privpass': 123
                    }
                },
                'timeout': 3600
            })
