# Copyright 2013 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import copy
import mock
import datetime
import json
from magellan.db.sqlalchemy import models
from magellan.common import exceptions as exc
from magellan.common import consts
from magellan.rpc.client import RpcClient
from magellan.tests.unit.api import base

JOB_ID = '123e4567-e89b-12d3-a456-426655440003'
JOB_DESC = "some description"
UPDATED_JOB_DESC = "updated description"

JOB_DB = models.AutoDiscoveryJob(
    id=JOB_ID,
    name='job1',
    description=JOB_DESC,
    method=consts.SCAN_METHOD_SEED_ROUTER,
    options={
        "seed_router": "192.168.1.1",
        "snmp": {
            "version": "v2c",
            "community": "public"
        }
    },
    timeout=3600,
    created_at=datetime.datetime(1970, 1, 1),
    updated_at=datetime.datetime(1970, 1, 1)
)

JOB = {
    'id': JOB_ID,
    'name': 'job1',
    'description': JOB_DESC,
    'method': consts.SCAN_METHOD_SEED_ROUTER,
    'options': {
        "seed_router": "192.168.1.1",
        "snmp": {
            "version": "v2c",
            "community": "public"
        }
    },
    'timeout': 3600,
    'created_at': '1970-01-01 00:00:00',
    'updated_at': '1970-01-01 00:00:00',
    "schedule": "* 0 * * 7 *"
}

JOB_PATCH = [
    {"op": "replace", "path": "/description", "value": UPDATED_JOB_DESC}
]
UPDATED_JOB_DB = copy.copy(JOB_DB)
UPDATED_JOB_DB['description'] = UPDATED_JOB_DESC


MOCK_JOB = mock.MagicMock(return_value=JOB_DB)
MOCK_JOBS = mock.MagicMock(return_value=[JOB_DB])
MOCK_UPDATED_JOB = mock.MagicMock(return_value=UPDATED_JOB_DB)
MOCK_DELETE = mock.MagicMock(return_value=None)
MOCK_EMPTY = mock.MagicMock(return_value=[])
MOCK_NOT_FOUND = mock.MagicMock(side_effect=exc.DBEntityNotFoundError())
MOCK_DUPLICATE = mock.MagicMock(side_effect=exc.DBDuplicateEntryError())
MOCK_EXECUTIONS = mock.MagicMock(return_value=[])


class TestWorkbooksController(base.APITest):
    @mock.patch.object(RpcClient, "job_get", MOCK_JOB)
    @mock.patch.object(RpcClient, "execution_get_all_by_job", MOCK_EXECUTIONS)
    def test_get(self):
        resp = self.app.get('/v1/jobs/%s' % JOB_ID)

        self.assertEqual(200, resp.status_int)
        self.assertEqual(JOB['name'], resp.json['name'])
        self.assertEqual(JOB['method'], resp.json['method'])

    @mock.patch.object(RpcClient, "job_get", MOCK_NOT_FOUND)
    def test_get_not_found(self):
        resp = self.app.get('/v1/jobs/%s' % JOB_ID, expect_errors=True)

        self.assertEqual(404, resp.status_int)

    @mock.patch.object(RpcClient, "job_get", MOCK_JOB)
    @mock.patch.object(RpcClient, "job_update", MOCK_UPDATED_JOB)
    def test_patch(self):
        resp = self.app.patch(
            '/v1/jobs/%s' % JOB_ID,
            json.dumps(JOB_PATCH),
            headers={'Content-Type': 'application/json'}
        )

        self.assertEqual(200, resp.status_int)
        self.assertEqual(JOB['name'], resp.json['name'])
        self.assertEqual(UPDATED_JOB_DESC, resp.json['description'])

    @mock.patch.object(RpcClient, "job_get", MOCK_NOT_FOUND)
    @mock.patch.object(RpcClient, "job_update", MOCK_NOT_FOUND)
    def test_patch_not_found(self):
        resp = self.app.patch(
            '/v1/jobs/%s' % JOB_ID,
            json.dumps(JOB_PATCH),
            headers={'Content-Type': 'application/json'},
            expect_errors=True
        )

        self.assertEqual(404, resp.status_int)

    @mock.patch.object(RpcClient, "job_create", MOCK_JOB)
    def test_post(self):
        resp = self.app.post(
            '/v1/jobs',
            json.dumps(JOB),
            headers={'Content-Type': 'application/json'}
        )

        self.assertEqual(201, resp.status_int)
        self.assertEqual(JOB['name'], resp.json['name'])
        self.assertEqual(JOB['method'], resp.json['method'])

    @mock.patch.object(RpcClient, "job_create", MOCK_DUPLICATE)
    def test_post_dup(self):
        resp = self.app.post(
            '/v1/jobs',
            json.dumps(JOB),
            headers={'Content-Type': 'application/json'},
            expect_errors=True
        )

        self.assertEqual(409, resp.status_int)

    @mock.patch.object(RpcClient, "job_delete", MOCK_DELETE)
    def test_delete(self):
        resp = self.app.delete('/v1/jobs/%s' % JOB_ID)

        self.assertEqual(204, resp.status_int)

    @mock.patch.object(RpcClient, "job_delete", MOCK_NOT_FOUND)
    def test_delete_not_found(self):
        resp = self.app.delete('/v1/jobs/%s' % JOB_ID, expect_errors=True)

        self.assertEqual(404, resp.status_int)

    @mock.patch.object(RpcClient, "job_get_all", MOCK_JOBS)
    def test_get_all(self):
        resp = self.app.get('/v1/jobs')

        self.assertEqual(200, resp.status_int)

        self.assertEqual(1, len(resp.json))
        self.assertEqual(JOB['name'], resp.json[0]['name'])
        self.assertEqual(JOB['description'],
                         resp.json[0]['description'])

    @mock.patch.object(RpcClient, "job_get_all", MOCK_EMPTY)
    def test_get_all_empty(self):
        resp = self.app.get('/v1/jobs')

        self.assertEqual(200, resp.status_int)

        self.assertEqual(0, len(resp.json))
