import datetime
import mock

from magellan.common import consts, exceptions
from magellan.db.sqlalchemy import api as db_api
from magellan.engine import discover
from magellan.tests import base as test_base
from pyautodiscover import Link

DISCOVERY_JOBS = [
    {
        'name': 'my_job1',
        'description': 'empty',
        'method': consts.SCAN_METHOD_SEED_ROUTER,
        'options': {
            'seed_router': '192.168.1.1',
            'snmp': {
                'version': 'v2c',
                'community': 'public'
            }
        },
        'timeout': 1800,
        'updated_at': None,
        'created_at': datetime.datetime(2017, 12, 1, 15, 0, 0)
    },
    {
        'name': 'my_job2',
        'description': 'my description',
        'method': consts.SCAN_METHOD_SEED_ROUTER,
        'options': {
            'seed_router': '192.168.1.1',
            'snmp': {
                'version': 'v1',
                'community': 'private'
            }
        },
        'timeout': 3600,
        'schedule': '0 */4 * * *',
        'updated_at': None,
        'created_at': datetime.datetime(2017, 12, 1, 15, 1, 0)
    },
]

DISCOVERY_EXECUTIONS = [
    {
        'job_id': None,
        'name': 'my_execution1',
        'description': 'my description',
        'status': consts.ExecutionStatus.running
    },
    {
        'job_id': None,
        'name': 'my_execution2',
        'description': 'my description',
        'status': consts.ExecutionStatus.running
    }
]


def add_job_id(data, job):
    copy = data.copy()
    copy['job_id'] = job['id']
    return copy


class DiscoveryExecutionTest(test_base.SqlAlchemyTest):
    @mock.patch.object(discover, 'SNMPBackend')
    @mock.patch.object(discover, 'info')
    @mock.patch.object(discover, 'discover')
    def test_execution_start(self, discover_func, info, backend_cls):
        discover_func.return_value = [
            Link('192.168.1.3', '192.168.1.4')
        ]
        info.return_value = 'foobar'

        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )
        discover._execution_start(execution)

        devices = db_api.device_get_all(execution['id'])
        connections = db_api.connection_get_all(execution['id'])

        self.assertEqual(
            [device.address for device in devices],
            ['192.168.1.3', '192.168.1.4']
        )
        self.assertEqual(
            [
                (connection.device1_id, connection.device2_id)
                for connection in connections
            ],
            [
                (devices[0].id, devices[1].id)
            ]
        )

    @mock.patch.object(discover, 'LOG')
    @mock.patch.object(discover, 'SNMPBackend')
    @mock.patch.object(discover, 'info')
    @mock.patch.object(discover, 'discover')
    def test_execution_start_deleted(self, discover_func, info, backend_cls,
                                     logger):
        discover.return_value = [
            Link('192.168.1.3', '192.168.1.4')
        ]
        info.return_value = 'foobar'

        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )
        db_api.execution_delete(execution.id)
        discover._execution_start(execution)

        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.device_get_all,
                          execution['id'])
        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.connection_get_all,
                          execution['id'])
        self.assertEqual(logger.info.call_count, 1)
