from magellan.common import exceptions
from magellan.db.sqlalchemy import api as db_api
from magellan.tests import base as test_base
from magellan.tests.unit.db.test_db_api import (
    DISCOVERY_JOBS,
    DISCOVERY_EXECUTIONS,
    add_job_id
)


class DiscoveryJobTest(test_base.SqlAlchemyTest):
    def test_create_discovery_job(self):
        created = db_api.job_create(DISCOVERY_JOBS[0])

        self.assertIsNotNone(created)

        self.assertEqual('my_job1', created['name'])

    def test_get_discovery_job(self):
        created = db_api.job_create(DISCOVERY_JOBS[0])

        fetched = db_api.job_get(created['id'])

        self.assertEqual(created['id'], fetched['id'])
        self.assertEqual(created['options'], fetched['options'])

    def test_get_all_discovery_jobs(self):
        j1 = db_api.job_create(DISCOVERY_JOBS[0])
        j2 = db_api.job_create(DISCOVERY_JOBS[1])

        fetched = db_api.job_get_all()

        self.assertEqual(len(fetched), 2)
        self.assertEqual(fetched[0]['name'], j1['name'])
        self.assertEqual(fetched[1]['name'], j2['name'])

    def test_update_discovery_job(self):
        created = db_api.job_create(DISCOVERY_JOBS[0])

        self.assertIsNotNone(created)

        DISCOVERY_JOBS[0]['description'] = 'not empty'
        DISCOVERY_JOBS[0]['schedule'] = '0 */4 * * *'

        updated = db_api.job_update(created['id'], DISCOVERY_JOBS[0])

        self.assertIsNotNone(updated)
        self.assertEqual(DISCOVERY_JOBS[0]['description'],
                         updated['description'])
        self.assertEqual(DISCOVERY_JOBS[0]['schedule'],
                         updated['schedule'])

    def test_delete_discovery_job(self):
        created = db_api.job_create(DISCOVERY_JOBS[0])

        self.assertIsNotNone(created)

        db_api.job_delete(created['id'])

        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.job_get,
                          created['id'])


class DiscoveryExecutionTest(test_base.SqlAlchemyTest):
    def test_create_discovery_execution(self):
        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )

        self.assertIsNotNone(execution)
        self.assertEqual('my_execution1', execution['name'])

    def test_get_discovery_execution(self):
        job = db_api.job_create(DISCOVERY_JOBS[0])
        created = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )
        fetched = db_api.execution_get(created['id'])

        self.assertEqual(created['id'], fetched['id'])

    def test_create_discovery_execution_with_devices(self):
        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )
        d1 = db_api.device_create({
            'execution_id': execution['id'],
            'address': '192.168.1.2',
            'details': '{}'
        })
        d2 = db_api.device_create({
            'execution_id': execution['id'],
            'address': '192.168.1.3',
            'details': '{}'
        })
        connection = db_api.connection_create({
            'execution_id': execution['id'],
            'device1_id': d1['id'],
            'device2_id': d2['id']
        })

        self.assertIsNotNone(execution)
        self.assertIsNotNone(d1)
        self.assertIsNotNone(d2)
        self.assertIsNotNone(connection)
        self.assertEqual(len(db_api.device_get_all(execution['id'])), 2)
        self.assertEqual(len(db_api.connection_get_all(execution['id'])), 1)

    def test_get_all_discovery_executions(self):
        j1 = db_api.job_create(DISCOVERY_JOBS[0])
        j2 = db_api.job_create(DISCOVERY_JOBS[1])
        e1 = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], j1)
        )
        e2 = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[1], j2)
        )

        fetched = db_api.execution_get_all()

        self.assertEqual(len(fetched), 2)
        self.assertEqual(fetched[0]['name'], e1['name'])
        self.assertEqual(fetched[1]['name'], e2['name'])

    def test_get_all_discovery_executions_by_job(self):
        j1 = db_api.job_create(DISCOVERY_JOBS[0])
        j2 = db_api.job_create(DISCOVERY_JOBS[1])
        e1 = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], j1)
        )
        db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[1], j2)
        )

        fetched = db_api.execution_get_all_by_job(j1['id'])

        self.assertEqual(len(fetched), 1)
        self.assertEqual(fetched[0]['name'], e1['name'])

    def test_delete_discovery_execution(self):
        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )

        self.assertIsNotNone(execution)

        db_api.execution_delete(execution['id'])

        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.execution_get,
                          execution['id'])

    def test_delete_discovery_execution_with_devices(self):
        job = db_api.job_create(DISCOVERY_JOBS[0])
        execution = db_api.execution_create(
            add_job_id(DISCOVERY_EXECUTIONS[0], job)
        )
        d1 = db_api.device_create({
            'execution_id': execution['id'],
            'address': '192.168.1.2',
            'details': '{}'
        })
        d2 = db_api.device_create({
            'execution_id': execution['id'],
            'address': '192.168.1.3',
            'details': '{}'
        })
        db_api.connection_create({
            'execution_id': execution['id'],
            'device1_id': d1['id'],
            'device2_id': d2['id']
        })

        db_api.execution_delete(execution['id'])

        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.execution_get,
                          execution['id'])
        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.device_get_all,
                          execution['id'])
        self.assertRaises(exceptions.DBEntityNotFoundError,
                          db_api.connection_get_all,
                          execution['id'])
