#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# pylint: disable=E1102
# disable warning about the constructors in the types imported from
# backends.snmp not being callable

from pyautodiscover import discover
from pyautodiscover.backends.snmp import SNMPBackend, SNMPInfo, AuthProto, \
                                         PrivProto, AuthInfo, PrivInfo
from ipaddress import ip_address
import argparse
import csv
import sys

import logging


logger = logging.getLogger()
handler = logging.StreamHandler()
formatter = logging.Formatter(
        '%(name)-12s %(levelname)-8s %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)


def ipaddress_parser(ip):
    try:
        return ip_address(ip)
    except ValueError:
        raise argparse.ArgumentTypeError(
                "{} is not a valid IPv4 address".format(ip))


def v1info(args):
    return SNMPInfo.SnmpV1(args.community)


def v2cinfo(args):
    return SNMPInfo.SnmpV2c(args.community)


def v3info(args):
    if args.auth_key is not None:
        ai = AuthInfo(AuthProto.SHA(), args.auth_key)
        if args.priv_key is not None:
            pi = PrivInfo(PrivProto.AES(), args.priv_key)
            return SNMPInfo.SnmpV3AuthPriv(args.secname, ai, pi)
        return SNMPInfo.SnmpV3AuthNoPriv(args.secname, ai)
    return SNMPInfo.SnmpV3NoAuthNoPriv(args.secname)


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        description="""
Python discovery script.
Use SNMP to retrieve and recursively walk through neighbours of a
target address by LLDP information.
        """)

    subparsers = parser.add_subparsers()

    parser_v1 = subparsers.add_parser('v1', help='Use SNMPv1')
    parser_v1.add_argument(
        "-c",
        "--community",
        type=str,
        default="public",
        help="community string, default is 'public'")
    parser_v1.set_defaults(snmpinfo=v1info)

    parser_v2c = subparsers.add_parser('v2c', help='Use SNMPv2c')
    parser_v2c.add_argument(
        "-c",
        "--community",
        type=str,
        default="public",
        help="community string, default is 'public'")
    parser_v2c.set_defaults(snmpinfo=v2cinfo)

    parser_v3 = subparsers.add_parser(
        'v3',
        help="Use SNMPv3 with optional SHA authentication and AES encryption")
    parser_v3.add_argument(
        "secname",
        type=str,
        help="SNMPv3 security user name")
    parser_v3.add_argument(
        "--auth-key",
        type=str,
        help="authentication key, enables authentication")
    parser_v3.add_argument(
        "--priv-key",
        type=str,
        help="private key, requires --auth-key to be set")
    parser_v3.set_defaults(snmpinfo=v3info)

    parser.add_argument(
        "address",
        metavar="address",
        type=ipaddress_parser,
        help="the address to start the discovery from")

    args = parser.parse_args()

    valueKeys = ["address", "neighbour"]
    outputStream = csv.DictWriter(sys.stdout, valueKeys, lineterminator="\n")
    outputStream.writeheader()

    counter = 0
    backend = SNMPBackend(args.snmpinfo(args))
    for link in discover(backend, args.address):
        counter += 1
        outputStream.writerow({
            "address": str(link.address_a) if link.address_a else '',
            "neighbour": str(link.address_b) if link.address_b else ''
        })

    if counter == 0:
        sys.exit(1)


if __name__ == "__main__":
    main()
