# -*- coding: utf-8 -*-
import pysnmp.hlapi as SNMP
import logging
import attr
from attr.validators import instance_of
from pysnmp.error import PySnmpError
from ipaddress import IPv4Address, ip_address
from sumtypes import sumtype, constructor, match
from pyautodiscover.backend import BackendInterface


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


@sumtype
class AuthProto:
    SHA = constructor()


@sumtype
class PrivProto:
    AES = constructor()


@attr.s
class AuthInfo:
    proto = attr.ib(validator=instance_of(AuthProto))
    password = attr.ib(validator=instance_of(str))


@attr.s
class PrivInfo:
    proto = attr.ib(validator=instance_of(PrivProto))
    password = attr.ib(validator=instance_of(str))


@sumtype
class SNMPInfo:
    SnmpV1 = constructor(community=attr.ib(validator=instance_of(str),
                                           default="public"))
    SnmpV2c = constructor(community=attr.ib(validator=instance_of(str),
                                            default="public"))
    SnmpV3NoAuthNoPriv = constructor(
                            secname=attr.ib(validator=instance_of(str))
                         )
    SnmpV3AuthNoPriv = constructor(
                            secname=attr.ib(validator=instance_of(str)),
                            authinfo=attr.ib(validator=instance_of(AuthInfo))
                       )
    SnmpV3AuthPriv = constructor(
                        secname=attr.ib(validator=instance_of(str)),
                        authinfo=attr.ib(validator=instance_of(AuthInfo)),
                        privinfo=attr.ib(validator=instance_of(PrivInfo))
                     )


class SNMPBackend(BackendInterface):

    def __init__(self, snmpinfo, port=161):
        """
        Creates an SNMP backend.
        :param port: SNMP port.
        :param snmpinfo: An instance of the SNMPInfo class
        """

        self.engine = SNMP.SnmpEngine()
        self.snmp_port = port
        self.snmpinfo = snmpinfo
        mib_url = 'http://mibs.snmplabs.com/asn1/@mib@'

        if not isinstance(port, int) or port <= 0:
            raise TypeError("port must be a positive integer")

        self.infoTypes = [
            SNMP.ObjectType(SNMP.ObjectIdentity(
                "SNMPv2-MIB", "sysName", 0
            ).addAsn1MibSource(mib_url)),
            SNMP.ObjectType(SNMP.ObjectIdentity(
                "SNMPv2-MIB", "sysDescr", 0
            ).addAsn1MibSource(mib_url))
        ]

        self.walkType = SNMP.ObjectType(SNMP.ObjectIdentity(
                            "LLDP-MIB", "lldpRemManAddrIfSubtype"
                        ).addAsn1MibSource(mib_url))

    def info(self, address):

        assert isinstance(address, IPv4Address)

        data = {}
        for objectType in self.infoTypes:
            try:
                errorIndication, errorStatus, errorIndex, entries = next(
                    SNMP.getCmd(self.engine,
                                _get_snmp_auth_data(self.snmpinfo),
                                SNMP.UdpTransportTarget((str(address),
                                                         self.snmp_port),
                                                        timeout=5,
                                                        retries=2),
                                SNMP.ContextData(),
                                objectType,
                                lexigraphicMode=False))
                if errorIndication:
                    err = '{} on {}'.format(errorIndication, str(address))
                    raise RuntimeError(err)
                elif errorStatus:
                    err = '{} at {}'.\
                        format(errorStatus.prettyPrint(),
                               (errorIndex and entries[errorIndex - 1][0])
                               or '?')
                    raise RuntimeError(err)
                else:
                    logger.info("SNMP get returned %s", str(entries))
                    for entry in entries:
                        name = entry[0].getMibSymbol()[1]
                        value = str(entry[1])
                        data[name] = value
            except PySnmpError as e:
                raise RuntimeError(str(e))

        return data

    def neighbours(self, address):

        assert isinstance(address, IPv4Address)

        nodes = set()
        try:
            commandGenerator = SNMP.nextCmd(
                self.engine,
                _get_snmp_auth_data(self.snmpinfo),
                SNMP.UdpTransportTarget(
                    (str(address), self.snmp_port), timeout=5, retries=2),
                SNMP.ContextData(),
                self.walkType,
                lexicographicMode=False)

            for (errorIndication,
                 errorStatus,
                 errorIndex,
                 entries) in commandGenerator:

                if errorIndication:
                    err = '{} on {}'.format(errorIndication, str(address))
                    raise RuntimeError(err)
                elif errorStatus:
                    err = '{} at {}'.\
                        format(errorStatus.prettyPrint(),
                               (errorIndex and entries[errorIndex - 1][0])
                               or '?')
                    raise RuntimeError(err)

                logger.info("SNMP get returned %s", str(entries))
                for entry in entries:
                    octets = str(entry[0]).split('.')
                    if len(octets) >= 4:
                        try:
                            node = ip_address(".".join(octets[-4:]))
                            nodes.add(node)
                        except ValueError:
                            pass

        except PySnmpError as e:
            raise RuntimeError(str(e))

        # If SNMP for some reason return the same IP that we're checking for
        # neighbours, then remove that from the set
        nodes.discard(address)
        return nodes


@match(SNMPInfo)
class _get_snmp_auth_data:
    def SnmpV1(community):
        return SNMP.CommunityData(community,
                                  communityName=community,
                                  mpModel=0)

    def SnmpV2c(community):
        return SNMP.CommunityData(community,
                                  communityName=community,
                                  mpModel=1)

    def SnmpV3NoAuthNoPriv(secname):
        return SNMP.UsmUserData(userName=secname)

    def SnmpV3AuthNoPriv(secname, authinfo):
        return SNMP.UsmUserData(userName=secname,
                                authKey=authinfo.password,
                                authProtocol=_get_auth_proto(authinfo.proto),
                                privProtocol=SNMP.usmNoPrivProtocol)

    def SnmpV3AuthPriv(secname, authinfo, privinfo):
        return SNMP.UsmUserData(userName=secname,
                                authKey=authinfo.password,
                                authProtocol=_get_auth_proto(authinfo.proto),
                                privKey=privinfo.password,
                                privProtocol=_get_priv_proto(privinfo.proto))


@match(AuthProto)
class _get_auth_proto:
    @staticmethod
    def SHA():
        return SNMP.usmHMACSHAAuthProtocol


@match(PrivProto)
class _get_priv_proto:
    @staticmethod
    def AES():
        return SNMP.usmAesCfb128Protocol
