# -*- coding: utf-8 -*-

from ipaddress import IPv4Address
from collections import namedtuple
from pyautodiscover.backend import BackendInterface
import logging


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Link(namedtuple('Link', ['address_a', 'address_b'])):

    def __hash__(self):
        return hash(frozenset((self.address_a, self.address_b)))

    def __eq__(self, other):
        return hash(self) == hash(other)

    def __lt__(self, other):
        return hash(self) < hash(other)


def info(backend, address):
    """
    Retrieves information regarding a device and returns it as a dict of values
    """

    if not isinstance(address, IPv4Address):
        raise TypeError("Expected address to be instance of IPv4Address")

    if not isinstance(backend, BackendInterface):
        raise TypeError("Expected backend to be instance of BackendInterface")

    try:
        return backend.info(address)
    except RuntimeError as e:
        # there was an error querying for info from the targeted device
        # log it and continue
        logger.exception(e)
        return {}


def discover(backend, base_address):
    """
    Runs discovery from the addresses given and yields Node discovered that
    contains address and parent as IPv4Address objects
    """

    if not isinstance(base_address, IPv4Address):
        raise TypeError("Expected address to be instance of IPv4Address")

    if not isinstance(backend, BackendInterface):
        raise TypeError("Expected backend to be instance of BackendInterface")

    yielded_links = set()
    checked = set()
    addresses_to_check = [base_address]

    for address in addresses_to_check:
        if address not in checked:
            checked.add(address)

            try:
                for neighbour in backend.neighbours(address):
                    new_link = Link(neighbour, address)
                    if new_link not in yielded_links:
                        yielded_links.add(new_link)
                        addresses_to_check.append(neighbour)
                        yield new_link
            except RuntimeError as e:
                # some other machine told us that this IP exists so this error
                # might be due to something else. Let's just ignore this error
                # and keep checking the machines that we find.
                logger.warning(str(e))
